@@ -245,7 +245,7 @@ _app_item_find_app(const struct app_item_context *ctxt, const char *category, co
   LauncherCategory *cat = _app_item_find_category(ctxt, category);
 
   if (cat)
-    n = launcher_category_get_children(cat);
+    n = launcher_category_get_applications(cat);
 
   if (!n)
     n = launcher_menu_get_applications(ctxt->parent->menu);
@@ -253,7 +253,7 @@ _app_item_find_app(const struct app_item_context *ctxt, const char *category, co
   for (; n != NULL; n = n->next)
   {
     LauncherApplication *app = n->data;
-    const char *cur = launcher_application_get_match_name(app);
+    const char *cur = launcher_application_get_unique_string(app);
     if (cur && strcmp(cur, match_name) == 0)
       return app;
   }
@@ -291,7 +291,7 @@ _app_glib_item_select(void *data, unsigned int len, const char **strings)
 {
   struct app_item_context *ctxt = data;
   LauncherApplication *app;
-  WnckApplication *running;
+  gboolean running;
   const char *category;
   const char *match_name;
   GError *err = NULL;
@@ -322,29 +322,15 @@ _app_glib_item_select(void *data, unsigned int len, const char **strings)
     return;
   }
 
-  running = launcher_application_get_wnckapp(app);
-  g_debug("app %s (%p, %s) WnckApplication=%p\n",
+  running = launcher_application_get_running(app);
+  g_debug("app %s (%p, %s)\n",
           launcher_application_get_name(app), app,
-          launcher_application_get_match_name(app), running);
+          launcher_application_get_unique_string(app));
   if (running)
   {
-    gulong xid = wnck_application_get_xid(running);
-    WnckWindow *win = wnck_window_get(xid);
-
-    g_debug("application %s is already running as %p, xid=%#lx, WnckWindow=%p",
-            launcher_application_get_name(app), running, xid, win);
-
-    if (win)
-      wnck_window_activate(win, GDK_CURRENT_TIME);
-    else
-    {
-      GList *n = wnck_application_get_windows(running);
-      for (; n != NULL; n = n->next)
-      {
-        win = n->data;
-        wnck_window_activate_transient(win, GDK_CURRENT_TIME);
-      }
-    }
+    g_debug("application %s is already running",
+            launcher_application_get_name(app));
+    launcher_application_show(app);
     gstuff_ecore_run_simple(_app_ecore_item_activated, ctxt);
     return;
   }
@@ -440,13 +426,13 @@ _app_item_is_running(struct app_item_context *ctxt)
   return 0;
 }
 
-static int
+static Eina_Bool
 _app_item_ecore_stop_anim_delayed(void *data)
 {
   struct app_item_context *ctxt = data;
   edje_object_play_set(ctxt->edje, 0);
   ctxt->timer.stop_anim = NULL;
-  return 0;
+  return ECORE_CALLBACK_CANCEL;
 }
 
 static void
@@ -458,7 +444,7 @@ _app_item_ecore_stop_anim_request(struct app_item_context *ctxt)
     (2.0, _app_item_ecore_stop_anim_delayed, ctxt);
 }
 
-static int
+static Eina_Bool
 _app_item_ecore_win_app_open(void *data, int type __UNUSED__, void *event)
 {
   struct app_item_context *ctxt = data;
@@ -466,7 +452,7 @@ _app_item_ecore_win_app_open(void *data, int type __UNUSED__, void *event)
   int play;
 
   if (match_name != ctxt->match_name) /* both are stringshared */
-      return 1;
+      return ECORE_CALLBACK_RENEW;
 
   play = edje_object_play_get(ctxt->edje);
   edje_object_play_set(ctxt->edje, 1);
@@ -477,10 +463,10 @@ _app_item_ecore_win_app_open(void *data, int type __UNUSED__, void *event)
   if (ctxt->pending_launch)
     launchfeedback_hide();
 
-  return 1;
+  return ECORE_CALLBACK_RENEW;
 }
 
-static int
+static Eina_Bool
 _app_item_ecore_win_app_close(void *data, int type __UNUSED__, void *event)
 {
   struct app_item_context *ctxt = data;
@@ -488,7 +474,7 @@ _app_item_ecore_win_app_close(void *data, int type __UNUSED__, void *event)
   int play;
 
   if (match_name != ctxt->match_name) /* both are stringshared */
-      return 1;
+      return ECORE_CALLBACK_RENEW;
 
   play = edje_object_play_get(ctxt->edje);
   edje_object_play_set(ctxt->edje, 1);
@@ -496,19 +482,19 @@ _app_item_ecore_win_app_close(void *data, int type __UNUSED__, void *event)
     _app_item_ecore_stop_anim_request(ctxt);
 
   edje_object_signal_emit(ctxt->edje, _EDJE_SIG_WIN_CLOSE, _EDJE_SIG_SRC);
-  return 1;
+  return ECORE_CALLBACK_RENEW;
 }
 
-static int
+static Eina_Bool
 _app_item_ecore_win_apps_changed(void *data, int type __UNUSED__, void *event)
 {
   struct app_item_common *common = data;
   const struct string_array *running_apps = event;
   common->running_apps = running_apps;
-  return 1;
+  return ECORE_CALLBACK_RENEW;
 }
 
-static int
+static Eina_Bool
 _app_item_ecore_win_focus_in(void *data, int type __UNUSED__, void *event __UNUSED__)
 {
   struct app_item_context *ctxt = data;
@@ -521,15 +507,15 @@ _app_item_ecore_win_focus_in(void *data, int type __UNUSED__, void *event __UNUS
 
   edje_object_play_set(ctxt->edje, 1);
 
-  return 1;
+  return ECORE_CALLBACK_RENEW;
 }
 
-static int
+static Eina_Bool
 _app_item_ecore_win_focus_out(void *data, int type __UNUSED__, void *event __UNUSED__)
 {
   struct app_item_context *ctxt = data;
   _app_item_ecore_stop_anim_request(ctxt);
-  return 1;
+  return ECORE_CALLBACK_RENEW;
 }
 
 static void
