@@ -181,7 +181,7 @@ _apps_glib_load_category(void *data, const char *category)
   }
   evas_object_size_hint_min_set(ctxt->grid, 0, 0);
 
-  lst = launcher_category_get_children(cat);
+  lst = launcher_category_get_applications(cat);
   len = g_slist_length(lst);
 
   Eina_Bool is_favorite = EINA_FALSE;
@@ -196,7 +196,7 @@ _apps_glib_load_category(void *data, const char *category)
     comment = launcher_application_get_comment(app);
     comment = !comment ? "" : comment;
     app_icon = launcher_application_get_icon_name(app);
-    match_name = launcher_application_get_match_name(app);
+    match_name = launcher_application_get_unique_string(app);
     match_name = !match_name ? "" : match_name;
     dfile = launcher_application_get_desktop_file(app);
     dfile = !dfile ? "" : dfile;
@@ -221,23 +221,6 @@ error:
     _apps_ecore_quit_show(ctxt, ctxt->category == ctxt->favorites_id);
 }
 
-static LauncherApplication *
-_apps_glib_app_find(const struct apps_context *ctxt, const char *desktop_file)
-{
-  const GSList *n;
-
-  n = launcher_menu_get_applications(ctxt->item.menu);
-  for (; n != NULL; n = n->next)
-  {
-    LauncherApplication *app = n->data;
-    const gchar *cur_desktop_file = launcher_application_get_desktop_file(app);
-    if (g_strcmp0(cur_desktop_file, desktop_file) == 0)
-      return app;
-  }
-
-  return NULL;
-}
-
 static LauncherApplication *
 _apps_glib_favorite_app_find(const struct apps_context *ctxt, const char *uid)
 {
@@ -261,16 +244,30 @@ _apps_glib_favorite_app_find(const struct apps_context *ctxt, const char *uid)
 
   desktop_file = launcher_favorites_get_string
     (ctxt->item.favs, uid, "desktop_file");
-  if (!desktop_file)
+  if (!desktop_file || !g_file_test(desktop_file, G_FILE_TEST_EXISTS))
   {
     g_warning("no desktop file for favorite id '%s'", uid);
+    g_free(desktop_file);
     return NULL;
   }
 
-  app = _apps_glib_app_find(ctxt, desktop_file);
-  if (!app)
-    g_warning("no application matches desktop file '%s', favorite id '%s'",
+  app = launcher_application_new_from_desktop_file(desktop_file);
+  if (!app) {
+    g_warning("could not create launcher app for desktop file '%s', favorite id '%s'",
               desktop_file, uid);
+    g_free(desktop_file);
+    return NULL;
+  }
+
+  /* If loading desktop failed, sometimes we'll get a valid, but 
+     empty LauncherApplication.  Check for that here. */
+  if (!launcher_application_get_desktop_file(app)) {
+    g_warning("could not create launcher app for desktop file '%s', favorite id '%s'",
+              desktop_file, uid);
+    g_free(desktop_file);
+    return NULL;
+  }
+
   g_free(desktop_file);
   return app;
 }
@@ -408,6 +405,7 @@ _apps_glib_load_gadgets(void *data)
   {
     GadgetInfo *info = g->data;
     Evas_Object *item;
+    char *gadget_name = NULL;
     char *icon = NULL;
     char *uid;
 
@@ -416,17 +414,19 @@ _apps_glib_load_gadgets(void *data)
 
     uid = efl_gadget_uid_make();
 
-    /* First check if there is an icon based on info->name.
+    /* First check if there is an icon based on the name of the gadget file.
         This might happen if we pre-ship some icons. */
-    icon = _get_icon(info->name);
+    gadget_name = g_path_get_basename(info->path);
+    icon = _get_icon(gadget_name);
     if (!icon && info->icon_name)
       icon = _get_icon(info->icon_name);
     if (!icon && info->icon)
     {
-      char *escaped = g_uri_escape_string(info->name, NULL, TRUE);
+      char *escaped = g_uri_escape_string(gadget_name, NULL, TRUE);
       icon = gstuff_pixbuf_to_file(info->icon, escaped);
       g_free(escaped);
     }
+    g_free(gadget_name);
 
     item = app_gadget_add
       (&ctxt->item, ctxt->category, info->name, info->description,
@@ -507,7 +507,7 @@ _apps_glib_load_favorites(void *data)
       app_icon_name = fav_uri ? launcher_favorites_get_string(ctxt->item.favs,
           uid, "icon") : launcher_application_get_icon_name(app);
       match_name = fav_uri ? "firefox" :
-        launcher_application_get_match_name(app);
+        launcher_application_get_unique_string(app);
       match_name = !match_name ? "" : match_name;
 
       icon_name = icon_path_get(app_icon_name, ctxt->item.item.icon_size);
@@ -842,13 +842,13 @@ _apps_exit_end(void *data, Evas_Object *o __UNUSED__, const char *emission __UNU
   edje_object_signal_emit(ctxt->edje, _EDJE_SIG_ENTER, _EDJE_SIG_SRC);
 }
 
-static int
+static Eina_Bool
 _apps_favorites_changed(void *data, int type __UNUSED__, void *event __UNUSED__)
 {
   struct apps_context *ctxt = data;
   if (ctxt->category == ctxt->favorites_id)
     gstuff_glib_run_simple(_apps_glib_load_favorites, ctxt);
-  return 1;
+  return ECORE_CALLBACK_RENEW;
 }
 
 static void
@@ -932,7 +932,7 @@ apps_add(Evas_Object *parent)
   ctxt->scroller = scroller;
   ctxt->grid = grid;
   ctxt->favorites_id = eina_stringshare_add(_favorites_id);
-  ctxt->gadgets_id = eina_stringshare_add(_gadgets_id);
+  ctxt->gadgets_id = eina_stringshare_add(_(_gadgets_id));
 
   _apps_context_set(apps, ctxt);
 
