@@ -209,7 +209,7 @@ _places_storage_ecore_item_error(void *data, unsigned int len, const char **stri
   uid = strings[0];
   message = strings[1];
   _places_storage_ecore_item_done(ctxt, uid);
-  error_show(uid);
+  error_show(message);
 }
 
 static Eina_Bool
