@@ -24,6 +24,7 @@
 
 #include <Elementary.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkx.h>
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
 #include <launcher/launcher.h>
@@ -104,7 +105,6 @@ struct win_context
     gulong sound_theme;
     gulong sound_enable;
     gulong app_open;
-    gulong app_close;
   } notify;
   struct
   {
@@ -422,7 +422,7 @@ _win_glib_apps_changed(struct win_context *ctxt)
   for (i = 0, n = lst; n != NULL; n = n->next)
   {
     LauncherApplication *app = n->data;
-    const char *match_name = launcher_application_get_match_name(app);
+    const char *match_name = launcher_application_get_unique_string(app);
     if (!match_name)
       continue;
 
@@ -434,24 +434,9 @@ _win_glib_apps_changed(struct win_context *ctxt)
 }
 
 static void
-_win_glib_app_open(LauncherSession *session __UNUSED__, LauncherApplication *app, struct win_context *ctxt)
+_win_glib_app_close(LauncherApplication *app, WnckApplication *wnckapp __UNUSED__, struct win_context *ctxt)
 {
-  const char *match_name = launcher_application_get_match_name(app);
-  if (!match_name)
-  {
-    g_debug("application opened, but no desktop file: %s, ignored.",
-            launcher_application_get_name(app));
-    return;
-  }
-
-  _win_glib_apps_changed(ctxt);
-  gstuff_ecore_run_string(_win_ecore_app_open, match_name);
-}
-
-static void
-_win_glib_app_close(LauncherSession *session __UNUSED__, LauncherApplication *app, struct win_context *ctxt)
-{
-  const char *match_name = launcher_application_get_match_name(app);
+  const char *match_name = launcher_application_get_unique_string(app);
   if (!match_name)
   {
     g_debug("application closed, but no desktop file: %s, ignored.",
@@ -463,6 +448,23 @@ _win_glib_app_close(LauncherSession *session __UNUSED__, LauncherApplication *ap
   gstuff_ecore_run_string(_win_ecore_app_close, match_name);
 }
 
+static void
+_win_glib_app_open(LauncherSession *session __UNUSED__, LauncherApplication *app, struct win_context *ctxt)
+{
+  const char *match_name = launcher_application_get_unique_string(app);
+  if (!match_name)
+  {
+    g_debug("application opened, but no desktop file: %s, ignored.",
+            launcher_application_get_name(app));
+    return;
+  }
+
+  g_signal_connect(app, "closed", G_CALLBACK(_win_glib_app_close), ctxt);
+
+  _win_glib_apps_changed(ctxt);
+  gstuff_ecore_run_string(_win_ecore_app_open, match_name);
+}
+
 static void
 _win_glib_stop(void *data)
 {
@@ -473,9 +475,6 @@ _win_glib_stop(void *data)
   if (ctxt->notify.app_open)
     g_signal_handler_disconnect(ctxt->session, ctxt->notify.app_open);
 
-  if (ctxt->notify.app_close)
-    g_signal_handler_disconnect(ctxt->session, ctxt->notify.app_close);
-
   if (ctxt->session)
     g_object_unref(ctxt->session);
 
@@ -605,10 +604,6 @@ _win_glib_start(void *data)
     (ctxt->session, "application-opened",
      G_CALLBACK(_win_glib_app_open), ctxt);
 
-  ctxt->notify.app_close = g_signal_connect
-    (ctxt->session, "application-closed",
-     G_CALLBACK(_win_glib_app_close), ctxt);
-
   if (!icon_path_init())
   {
     g_critical("could not init icon path cache.");
@@ -800,7 +795,7 @@ _win_glib_quit(void)
 #endif
 }
 
-static int
+static Eina_Bool
 _win_icon_info_enter(void *data, int type __UNUSED__, void *event)
 {
   struct win_context *ctxt = data;
@@ -817,10 +812,10 @@ _win_icon_info_enter(void *data, int type __UNUSED__, void *event)
   msg->str[1] = (char *)ev->comment;
   edje_object_message_send(ctxt->edje, EDJE_MESSAGE_STRING_SET, 0, msg);
 
-  return 1;
+  return ECORE_CALLBACK_RENEW;
 }
 
-static int
+static Eina_Bool
 _win_icon_info_leave(void *data, int type __UNUSED__, void *event __UNUSED__)
 {
   struct win_context *ctxt = data;
@@ -828,7 +823,7 @@ _win_icon_info_leave(void *data, int type __UNUSED__, void *event __UNUSED__)
   edje_object_part_text_set(ctxt->edje, _EDJE_PART_ICON_NAME, "");
   edje_object_part_text_set(ctxt->edje, _EDJE_PART_ICON_COMMENT, "");
   edje_object_signal_emit(ctxt->edje, _EDJE_SIG_ICON_LEAVE, _EDJE_SIG_SRC);
-  return 1;
+  return ECORE_CALLBACK_RENEW;
 }
 
 static void
@@ -837,6 +832,99 @@ _win_edje_debug(void *data __UNUSED__, Evas_Object *o __UNUSED__, const char *si
   fprintf(stderr, "EDJE-SIGNAL-DEBUG: '%s'\n", source);
 }
 
+static void
+_time_set(Evas_Object *win, unsigned int timestamp)
+{
+  Ecore_X_Window xwin;
+  GdkWindow *gdkwin, *topwin;
+  xwin = elm_win_xwindow_get(win);
+  if (!xwin)
+    return;
+  gdkwin = gdk_window_foreign_new(xwin);
+  if (!gdkwin)
+    return;
+  topwin = gdkwin;
+  while (gdk_window_get_parent(topwin))
+  {
+    topwin = gdk_window_get_parent(topwin);
+    topwin = gdk_window_get_toplevel(topwin);
+  }
+  gdk_x11_window_set_user_time(topwin, timestamp);
+  g_object_unref(gdkwin);
+}
+
+static Eina_Bool
+_time_kdown(void *data, int type, void *ev)
+{
+  Evas_Event_Key_Down *event = (Evas_Event_Key_Down *)ev;
+  struct win_context *ctxt = (struct win_context *)data;
+  _time_set(ctxt->win, event->timestamp);
+  return ECORE_CALLBACK_RENEW;
+}
+
+static Eina_Bool
+_time_kup(void *data, int type, void *ev)
+{
+  Evas_Event_Key_Up *event = (Evas_Event_Key_Up *)ev;
+  struct win_context *ctxt = (struct win_context *)data;
+  _time_set(ctxt->win, event->timestamp);
+  return ECORE_CALLBACK_RENEW;
+}
+
+static Eina_Bool
+_time_mdown(void *data, int type, void *ev)
+{
+  Evas_Event_Mouse_Down *event = (Evas_Event_Mouse_Down *)ev;
+  struct win_context *ctxt = (struct win_context *)data;
+  _time_set(ctxt->win, event->timestamp);
+  return ECORE_CALLBACK_RENEW;
+}
+
+static Eina_Bool
+_time_mup(void *data, int type, void *ev)
+{
+  Evas_Event_Mouse_Up *event = (Evas_Event_Mouse_Up *)ev;
+  struct win_context *ctxt = (struct win_context *)data;
+  _time_set(ctxt->win, event->timestamp);
+  return ECORE_CALLBACK_RENEW;
+}
+
+static Eina_Bool
+_time_mmove(void *data, int type, void *ev)
+{
+  Evas_Event_Mouse_Move *event = (Evas_Event_Mouse_Move *)ev;
+  struct win_context *ctxt = (struct win_context *)data;
+  _time_set(ctxt->win, event->timestamp);
+  return ECORE_CALLBACK_RENEW;
+}
+
+static Eina_Bool
+_time_min(void *data, int type, void *ev)
+{
+  Evas_Event_Mouse_In *event = (Evas_Event_Mouse_In *)ev;
+  struct win_context *ctxt = (struct win_context *)data;
+  _time_set(ctxt->win, event->timestamp);
+  return ECORE_CALLBACK_RENEW;
+}
+
+static Eina_Bool
+_time_mout(void *data, int type, void *ev)
+{
+  Evas_Event_Mouse_Out *event = (Evas_Event_Mouse_Out *)ev;
+  struct win_context *ctxt = (struct win_context *)data;
+  _time_set(ctxt->win, event->timestamp);
+  return ECORE_CALLBACK_RENEW;
+}
+
+static Eina_Bool
+_time_mwheel(void *data, int type, void *ev)
+{
+  Evas_Event_Mouse_Wheel *event = (Evas_Event_Mouse_Wheel *)ev;
+  struct win_context *ctxt = (struct win_context *)data;
+  _time_set(ctxt->win, event->timestamp);
+  return ECORE_CALLBACK_RENEW;
+}
+
 Evas_Object *
 win_add(void)
 {
@@ -926,6 +1014,17 @@ win_add(void)
   evas_object_event_callback_add
     (layout, EVAS_CALLBACK_KEY_DOWN, _win_key_down, ctxt);
 
+  /* Tell the window manager the timestamp of any user events.
+     This fixes focus issues when launching apps. */
+  ecore_event_handler_add(ECORE_EVENT_KEY_DOWN, _time_kdown, ctxt);
+  ecore_event_handler_add(ECORE_EVENT_KEY_UP, _time_kup, ctxt);
+  ecore_event_handler_add(ECORE_EVENT_MOUSE_BUTTON_DOWN, _time_mdown, ctxt);
+  ecore_event_handler_add(ECORE_EVENT_MOUSE_BUTTON_UP, _time_mup, ctxt);
+  ecore_event_handler_add(ECORE_EVENT_MOUSE_MOVE, _time_mmove, ctxt);
+  ecore_event_handler_add(ECORE_EVENT_MOUSE_IN, _time_min, ctxt);
+  ecore_event_handler_add(ECORE_EVENT_MOUSE_OUT, _time_mout, ctxt);
+  ecore_event_handler_add(ECORE_EVENT_MOUSE_WHEEL, _time_mwheel, ctxt);
+
   key = edje_object_data_get(edje, "contents_anim");
   if (key && atoi(key))
     ctxt->contents_anim = 1;
